@@ -33,6 +33,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "extend.h"
 #ifdef WITH_DNA
 #include "../dna/wfa2_test.h"
+#include "../dna/ksw2_extension.h"
 #endif
 #include "../util/algo/radix_sort.h"
 #include "target.h"
@@ -58,7 +59,7 @@ TextBuffer* pipeline_short(BlockId query, Search::Hit* begin, Search::Hit* end,
 
 }
 
-#ifndef EXTRA
+#if !defined(EXTRA) || defined(WITH_DNA)
 #define OLD
 #endif
 
@@ -197,7 +198,7 @@ TextBuffer* legacy_pipeline(const HitIterator::Hits& hits, Search::Config& cfg,
 
 	QueryMapper *mapper = new ExtensionPipeline::BandedSwipe::Pipeline(hits.query, hits.begin, hits.end, dp_stat, cfg);
 
-	task_timer timer("Initializing mapper", UINT_MAX);
+	TaskTimer timer("Initializing mapper", UINT_MAX);
 	mapper->init();
 	timer.finish();
 	mapper->run(stat, cfg);
@@ -253,7 +254,7 @@ bool align_worker(HitIterator* hit_it, ThreadPool::TaskSet* task_set, Search::Co
 
 			pair<vector<Extension::Match>, Extension::Stats> matches =
 #ifdef WITH_DNA
-				align_mode.mode == Align_mode::blastn ? WaveExtension::extend(*cfg, h->query) :
+           align_mode.mode == AlignMode::blastn ? Dna::extend(*cfg, cfg->query->seqs()[h->query]) :
 #endif
 				Extension::extend(h->query, h->begin, h->end, *cfg, stat, parallel ? DP::Flags::PARALLEL : DP::Flags::NONE);
 			TextBuffer* buf = cfg->blocked_processing ? Extension::generate_intermediate_output(matches.first, h->query, *cfg) : Extension::generate_output(matches.first, matches.second, h->query, stat, *cfg);
@@ -273,8 +274,8 @@ bool align_worker(HitIterator* hit_it, ThreadPool::TaskSet* task_set, Search::Co
 				}
 			}
 			output_sink->push(h->query, buf);
-			if (config.swipe_all && !config.no_heartbeat && (h->query % 100 == 0))
-				log_stream << "Queries = " << h->query << std::endl;
+			//if (config.swipe_all && !config.no_heartbeat && (h->query % 100 == 0))
+				//log_stream << "Queries = " << h->query << std::endl;
 
 		}
 
@@ -292,7 +293,7 @@ void align_queries(Consumer* output_file, Search::Config& cfg)
 	const int64_t mem_limit = Util::String::interpret_number(config.memory_limit.get("16G"));
 
 	pair<BlockId, BlockId> query_range;
-	task_timer timer(nullptr, 3);
+	TaskTimer timer(nullptr, 3);
 
 	if (!cfg.blocked_processing && !cfg.iterated())
 		cfg.db->init_random_access(cfg.current_query_block, 0, false);
@@ -312,10 +313,14 @@ void align_queries(Consumer* output_file, Search::Config& cfg)
 		cfg.seed_hit_buf->load(std::min(mem_limit - res_size, config.trace_pt_fetch_size));
 
 		if (res_size + last_size > mem_limit)
-			message_stream << "Warning: resident size (" << (res_size + last_size) << ") exceeds memory limit." << std::endl;
+			log_stream << "Warning: resident size (" << (res_size + last_size) << ") exceeds memory limit." << std::endl;
 
 		timer.go("Sorting trace points");
+#ifdef NDEBUG
 		ips4o::parallel::sort(hit_buf->begin(), hit_buf->end(), std::less<Search::Hit>(), config.threads_);
+#else
+		std::sort(hit_buf->begin(), hit_buf->end());
+#endif
 		statistics.inc(Statistics::TIME_SORT_SEED_HITS, timer.microseconds());
 
 #ifndef OLD
@@ -325,10 +330,10 @@ void align_queries(Consumer* output_file, Search::Config& cfg)
 
 		timer.go("Computing alignments");
 		HitIterator hit_it(query_range.first, query_range.second, hit_buf->data(), hit_buf->data() + hit_buf->size());
-		OutputWriter writer{ output_file };
+        OutputWriter writer{output_file, (*cfg.output_format == OutputFormat::json) ? ',' : char(0)};
 		output_sink.reset(new ReorderQueue<TextBuffer*, OutputWriter>(query_range.first, writer));
 		unique_ptr<thread> heartbeat;
-		if (config.verbosity >= 3 && config.load_balancing == Config::query_parallel && !config.no_heartbeat && !config.swipe_all)
+		if (config.verbosity >= 3 && config.load_balancing == Config::query_parallel && !config.swipe_all && config.heartbeat)
 			heartbeat.reset(new thread(heartbeat_worker, query_range.second, &cfg));
 		size_t n_threads = config.threads_align == 0 ? config.threads_ : config.threads_align;
 		if (config.load_balancing == Config::target_parallel || (config.swipe_all && (cfg.target->seqs().size() >= cfg.query->seqs().size())))
@@ -338,7 +343,7 @@ void align_queries(Consumer* output_file, Search::Config& cfg)
 		};
 #ifndef OLD
 		cfg.thread_pool.reset(new ThreadPool(task));
-		cfg.thread_pool->run(n_threads, !config.no_heartbeat);
+		cfg.thread_pool->run(n_threads, false);
 		cfg.thread_pool->join();
 #else
 		cfg.thread_pool.reset(new ThreadPool ());
