@@ -227,7 +227,7 @@ pair<vector<Match>, Stats> extend(
 				aligned_targets = move(v.first);
 
 			i0 = i1;
-			i1 += std::min((ptrdiff_t)std::min(chunk_size, MAX_CHUNK_SIZE), l.target_scores.cend() - i1);
+			i1 += std::min((ptrdiff_t)chunk_size, l.target_scores.cend() - i1);
 			previous_tail_score = tail_score;
 			if (new_hits)
 				tail_score = (i1 - 1)->score;
@@ -252,7 +252,7 @@ pair<vector<Match>, Stats> extend(
 }
 
 pair<vector<Match>, Stats> extend(BlockId query_id, Search::Hit* begin, Search::Hit* end, const Search::Config &cfg, Statistics &stat, DP::Flags flags) {
-	task_timer timer(flag_any(flags, DP::Flags::PARALLEL) ? config.target_parallel_verbosity : UINT_MAX);
+	TaskTimer timer(flag_any(flags, DP::Flags::PARALLEL) ? config.target_parallel_verbosity : UINT_MAX);
 	timer.go("Loading seed hits");
 	SeedHitList l = load_hits(begin, end, cfg.target->seqs());
 	stat.inc(Statistics::TARGET_HITS0, l.target_block_ids.size());
