@@ -26,6 +26,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "../output/daa/daa_write.h"
 #include "../util/sequence/sequence.h"
 
+
 using std::vector;
 
 namespace Extension {
@@ -37,7 +38,7 @@ TextBuffer* generate_output(vector<Match> &targets, const Extension::Stats& stat
 	std::unique_ptr<OutputFormat> f(cfg.output_format->clone());
 	size_t seek_pos = 0;
 	unsigned n_hsp = 0, hit_hsps = 0;
-	Output::Info info{ cfg.query->seq_info(query_block_id), !targets.empty(), cfg.db.get(), *out, stats };
+	Output::Info info{ cfg.query->seq_info(query_block_id), targets.empty(), cfg.db.get(), *out, stats };
 	TranslatedSequence query = query_seqs.translated_seq(align_mode.query_translated ? cfg.query->source_seqs()[query_block_id] : query_seqs[query_block_id], query_block_id * align_mode.query_contexts);
 	const char *query_title = cfg.query->ids()[query_block_id];
 	const double query_self_aln_score = cfg.query->has_self_aln() ? cfg.query->self_aln_score(query_block_id) : 0.0;
@@ -51,7 +52,7 @@ TextBuffer* generate_output(vector<Match> &targets, const Extension::Stats& stat
 	}
 	else if (aligned || config.report_unaligned)
 		f->print_query_intro(info);
-			
+
 	for (int i = 0; i < (int)targets.size(); ++i) {
 
 		if (targets[i].hsp.empty())
