@@ -1,4 +1,4 @@
-/****
+ /****
 DIAMOND protein aligner
 Copyright (C) 2013-2022 Max Planck Society for the Advancement of Science e.V.
                         Benjamin Buchfink
@@ -21,6 +21,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****/
 
 #include <memory>
+#include <iostream>
 #include <cstdlib>
 #include <sys/stat.h>
 #include <exception>
@@ -50,12 +51,12 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 using std::thread;
 using std::stringstream;
 using std::endl;
-using std::cerr;
 using std::ostream;
-using std::cout;
 using std::unique_ptr;
 using std::string;
 using std::pair;
+using std::cout;
+using std::cerr;
 
 const EMap<Sensitivity> EnumTraits<Sensitivity>::to_string = {
 	{ Sensitivity::FASTER, "faster" },
@@ -96,15 +97,16 @@ pair<double, int> block_size(int64_t memory_limit, Sensitivity s, bool lin) {
 	const double m = (double)memory_limit / 1e9;
 	const int min = std::max(Search::sensitivity_traits[(int)align_mode.sequence_type].at(s).minimizer_window, 1),
 		c = m < 40.0 && s <= Sensitivity::MORE_SENSITIVE && min == 1 ? 4 : 1;
+	const double min_factor = std::min(1 / (double)min * 2, 1.0);
 	const double max = s <= Sensitivity::DEFAULT ? 12.0 :
 		(s <= Sensitivity::MORE_SENSITIVE ? 4.0 : 0.4);
-	double b = m / (18.0 / c / min + 2.0);
-	if (b > 4)
+	double b = m / (18.0 * min_factor / c + 2.0);
+	/*if (b > 4)
 		b = floor(b);
 	else if (b > 0.4)
 		b = floor(b * 10) / 10;
 	else
-		b = floor(b * 1000) / 1000;
+		b = floor(b * 1000) / 1000;*/
 	if (!config.no_block_size_limit && !lin)
 		b = std::min(b, max);
 	if (s >= Sensitivity::VERY_SENSITIVE)
@@ -114,61 +116,14 @@ pair<double, int> block_size(int64_t memory_limit, Sensitivity s, bool lin) {
 	return { std::max(b, 0.001), c };
 }
 
-void print_warnings() {
-	if (config.sensitivity >= Sensitivity::VERY_SENSITIVE || config.verbosity == 0 || config.swipe_all)
-		return;
-	if (config.command != Config::blastp && config.command != Config::blastx && config.command != Config::blastn)
-		return;
-	const double ram = total_ram();
-	unsigned b = 2, c = 4;
-	stringstream msg;
-	if (config.sensitivity >= Sensitivity::SENSITIVE) {
-		if (ram >= 63) {
-			c = 1;
-			if(c < config.lowmem_)
-				msg << "use this parameter for better performance: -c1";
-		}
-	}
-	else {
-		if (ram >= 511) {
-			b = 12;
-			c = 1;
-		}
-		else if (ram >= 255) {
-			b = 8;
-			c = 1;
-		}
-		else if (ram >= 127) {
-			b = 5;
-			c = 1;
-		}
-		else if (ram >= 63) {
-			b = 6;
-		}
-		else if (ram >= 31) {
-			b = 4;
-		}
-		if ((b > 2 && b > config.chunk_size) || c < config.lowmem_) {
-			msg << "increase the block size for better performance using these parameters : -b" << b;
-			if (c != 4)
-				msg << " -c" << c;
-		}
-	}
-	if (!msg.str().empty()) {
-		set_color(Color::YELLOW, true);
-		cerr << "The host system is detected to have " << (size_t)ram << " GB of RAM. It is recommended to " << msg.str() << endl;
-		reset_color(true);
-	}
-}
-
-template<typename _t>
-_t set_string_option(const string& s, const string& name, const vector<pair<string, _t>>& values) {
+template<typename T>
+T set_string_option(const string& s, const string& name, const vector<pair<string, T>>& values) {
 	if (s.empty())
-		return (_t)0;
+		return (T)0;
 	for (auto& i : values)
 		if (s == i.first)
 			return i.second;
-	throw std::runtime_error("Invalid argument for option " + name + ". Allowed values are:" + std::accumulate(values.begin(), values.end(), string(), [](const string& s, const pair<string, _t>& v) { return s + ' ' + v.first; }));
+	throw std::runtime_error("Invalid argument for option " + name + ". Allowed values are:" + std::accumulate(values.begin(), values.end(), string(), [](const string& s, const pair<string, T>& v) { return s + ' ' + v.first; }));
 }
 
 void Config::set_sens(Sensitivity sens) {
@@ -195,8 +150,8 @@ Compressor Config::compressor() const
 
 Config::Config(int argc, const char **argv, bool check_io, CommandLineParser& parser)
 {
-	parser.add_command("makedb", "Build DIAMOND database from a FASTA file", makedb)
-		.add_command("prepdb", "Prepare BLAST or FASTA database for use with Diamond", prep_db)
+   parser.add_command("makedb", "Build DIAMOND database from a FASTA file", makedb)
+		.add_command("prepdb", "Prepare BLAST database for use with Diamond", prep_db)
 		.add_command("blastp", "Align amino acid query sequences against a protein reference database", blastp)
 		.add_command("blastx", "Align DNA query sequences against a protein reference database", blastx)
 		.add_command("cluster", "Cluster protein sequences", cluster)
@@ -244,44 +199,77 @@ Config::Config(int argc, const char **argv, bool check_io, CommandLineParser& pa
 		.add_command("listseeds", "", LIST_SEEDS)
 		.add_command("index-fasta", "", INDEX_FASTA)
 		.add_command("fetch-seq", "", FETCH_SEQ)
-        .add_command("blastn", "Align DNA query sequences against a DNA reference database", blastn)
+		.add_command("blastn", "Align DNA query sequences against a DNA reference database", blastn)
 		.add_command("length-sort", "", LENGTH_SORT)
+		.add_command("wc", "", WORD_COUNT)
+		.add_command("cut", "", CUT)
+		.add_command("model-seqs", "", MODEL_SEQS)
 #endif
 		;
 
-	auto& general = parser.add_group("General options", { makedb, blastp, blastx, cluster, view, prep_db, getseq, dbinfo, makeidx, CLUSTER_REALIGN, GREEDY_VERTEX_COVER, DEEPCLUST, RECLUSTER, MERGE_DAA, LINCLUST });
+	auto& general = parser.add_group("General options", { makedb, blastp, blastx, cluster, view, prep_db, getseq, dbinfo, makeidx, CLUSTER_REALIGN, GREEDY_VERTEX_COVER, DEEPCLUST, RECLUSTER, MERGE_DAA, LINCLUST, CLUSTER_REASSIGN });
 	general.add()
 		("threads", 'p', "number of CPU threads", threads_)
-		("db", 'd', "database file", database)
-		("out", 'o', "output file", output_file)
 		("verbose", 'v', "verbose console output", verbose)
 		("log", 0, "enable debug log", debug_log)
 		("quiet", 0, "disable console output", quiet)
+		("tmpdir", 't', "directory for temporary files", tmpdir);
+
+	auto& general_db = parser.add_group("General/database options", { makedb, blastp, blastx, cluster, prep_db, getseq, dbinfo, makeidx, CLUSTER_REALIGN, GREEDY_VERTEX_COVER, DEEPCLUST, RECLUSTER, LINCLUST, CLUSTER_REASSIGN });
+	general_db.add()
+		("db", 'd', "database file", database);
+
+	auto& general_out = parser.add_group("General/output", { blastp, blastx, cluster, view, getseq, CLUSTER_REALIGN, GREEDY_VERTEX_COVER, DEEPCLUST, RECLUSTER, MERGE_DAA, LINCLUST, CLUSTER_REASSIGN });
+	general_out.add()
+		("out", 'o', "output file", output_file);
+
+	auto& general_out2 = parser.add_group("General/output2", { blastp, blastx, cluster, view, CLUSTER_REALIGN, GREEDY_VERTEX_COVER, DEEPCLUST, RECLUSTER, LINCLUST, CLUSTER_REASSIGN });
+	general_out2.add()
 		("header", 0, "Use header lines in tabular output format (0/simple/verbose).", output_header, Option<vector<string>>(), 0);
 	
     string dbstring;
 	auto& makedb_opt = parser.add_group("Makedb options", { makedb, MERGE_DAA });
 	makedb_opt.add()
-		("in", 0, "input reference file in FASTA format", input_ref_file)
+		("in", 0, "input reference file in FASTA format/input DAA files for merge-daa", input_ref_file);
+
+	auto& makedb_tax_opt = parser.add_group("Makedb/taxon options", { makedb });
+	makedb_tax_opt.add()
 		("taxonmap", 0, "protein accession to taxid mapping file", prot_accession2taxid)
 		("taxonnodes", 0, "taxonomy nodes.dmp from NCBI", nodesdmp)
 		("taxonnames", 0, "taxonomy names.dmp from NCBI", namesdmp);
 
-	auto& align_clust = parser.add_group("Aligner/Clustering options", { blastp, blastx, cluster, RECLUSTER, CLUSTER_REASSIGN, DEEPCLUST, CLUSTER_REALIGN, LINCLUST });
-	align_clust.add()
-		("evalue", 'e', "maximum e-value to report alignments (default=0.001)", max_evalue, 0.001)
-		("tmpdir", 't', "directory for temporary files", tmpdir)
+	auto& align_clust_realign = parser.add_group("Aligner/Clustering/Realign options", { blastp, blastx, cluster, RECLUSTER, CLUSTER_REASSIGN, DEEPCLUST, CLUSTER_REALIGN, LINCLUST });
+	align_clust_realign.add()
 		("comp-based-stats", 0, "composition based statistics mode (0-4)", comp_based_stats, 1u)
 		("masking", 0, "masking algorithm (none, seg, tantan=default)", masking_)
-		("soft-masking", 0, "soft masking", soft_masking)
-		("motif-masking", 0, "softmask abundant motifs (0/1)", motif_masking)
-		("approx-id", 0, "minimum approx. identity% to report an alignment/to cluster sequences", approx_min_id)
-		("ext", 0, "Extension mode (banded-fast/banded-slow/full)", ext_)
-		("memory-limit", 'M', "Memory limit in GB (default = 16G)", memory_limit)
+		("soft-masking", 0, "soft masking (none=default, seg, tantan)", soft_masking)
 		("mmseqs-compat", 0, "", mmseqs_compat)
 		("no-block-size-limit", 0, "", no_block_size_limit);
 
-	auto& aligner = parser.add_group("Aligner options", { blastp, blastx, makeidx, CLUSTER_REASSIGN, view });
+	auto& align_clust = parser.add_group("Aligner/Clustering options", { blastp, blastx, cluster, RECLUSTER, CLUSTER_REASSIGN, DEEPCLUST, LINCLUST });
+	align_clust.add()		
+		("evalue", 'e', "maximum e-value to report alignments (default=0.001)", max_evalue, 0.001)
+		("motif-masking", 0, "softmask abundant motifs (0/1)", motif_masking)
+		("approx-id", 0, "minimum approx. identity% to report an alignment/to cluster sequences", approx_min_id)
+		("ext", 0, "Extension mode (banded-fast/banded-slow/full)", ext_);
+
+	auto& aligner_view = parser.add_group("Aligner/view options", { blastp, blastx, view });
+	aligner_view.add()
+		("max-target-seqs", 'k', "maximum number of target sequences to report alignments for (default=25)", max_target_seqs_)
+		("top", 0, "report alignments within this percentage range of top alignment score (overrides --max-target-seqs)", toppercent, 100.0);
+
+	auto& aligner_sens = parser.add_group("Aligner/sens options", { blastp, blastx, makeidx });
+	aligner_sens.add()
+		("faster", 0, "enable faster mode", mode_faster)
+		("fast", 0, "enable fast mode", mode_fast)
+		("mid-sensitive", 0, "enable mid-sensitive mode", mode_mid_sensitive)
+		("sensitive", 0, "enable sensitive mode)", mode_sensitive)
+		("more-sensitive", 0, "enable more sensitive mode", mode_more_sensitive)
+		("very-sensitive", 0, "enable very sensitive mode", mode_very_sensitive)
+		("ultra-sensitive", 0, "enable ultra sensitive mode", mode_ultra_sensitive)
+		("shapes", 's', "number of seed shapes (default=all available)", shapes);
+
+	auto& aligner = parser.add_group("Aligner options", { blastp, blastx });
 	aligner.add()
 		("query", 'q', "input query file", query_file)
 		("strand", 0, "query strands to search (both/minus/plus)", query_strands, string("both"))
@@ -290,8 +278,6 @@ Config::Config(int argc, const char **argv, bool check_io, CommandLineParser& pa
 		("unfmt", 0, "format of unaligned query file (fasta/fastq)", unfmt, string("fasta"))
 		("alfmt", 0, "format of aligned query file (fasta/fastq)", alfmt, string("fasta"))
 		("unal", 0, "report unaligned queries (0=no, 1=yes)", report_unaligned, -1)
-		("max-target-seqs", 'k', "maximum number of target sequences to report alignments for (default=25)", max_target_seqs_)
-		("top", 0, "report alignments within this percentage range of top alignment score (overrides --max-target-seqs)", toppercent, 100.0)
 		("max-hsps", 0, "maximum number of HSPs per target sequence to report for each query (default=1)", max_hsps, 1u)
 		("range-culling", 0, "restrict hit culling to overlapping query ranges", query_range_culling)
 		("compress", 0, "compression for output files (0=none, 1=gzip, zstd)", compression)
@@ -299,13 +285,6 @@ Config::Config(int argc, const char **argv, bool check_io, CommandLineParser& pa
 		("id", 0, "minimum identity% to report an alignment", min_id)
 		("query-cover", 0, "minimum query cover% to report an alignment", query_cover)
 		("subject-cover", 0, "minimum subject cover% to report an alignment", subject_cover)
-		("faster", 0, "enable faster mode", mode_faster)
-		("fast", 0, "enable fast mode", mode_fast)
-		("mid-sensitive", 0, "enable mid-sensitive mode", mode_mid_sensitive)
-		("sensitive", 0, "enable sensitive mode)", mode_sensitive)
-		("more-sensitive", 0, "enable more sensitive mode", mode_more_sensitive)
-		("very-sensitive", 0, "enable very sensitive mode", mode_very_sensitive)
-		("ultra-sensitive", 0, "enable ultra sensitive mode", mode_ultra_sensitive)
 		("swipe", 0, "exhaustive alignment against all database sequences", swipe_all)
 		("iterate", 0, "iterated search with increasing sensitivity", iterate, Option<vector<string>>(), 0)
 		("global-ranking", 'g', "number of targets for global ranking", global_ranking_targets)
@@ -334,8 +313,11 @@ Config::Config(int argc, const char **argv, bool check_io, CommandLineParser& pa
 \t5   = BLAST XML\n\
 \t6   = BLAST tabular\n\
 \t100 = DIAMOND alignment archive (DAA)\n\
-\t101 = SAM\n\n\
-\tValue 6 may be followed by a space-separated list of these keywords:\n\n\
+\t101 = SAM\n\
+\t102 = Taxonomic classification\n\
+\t103 = PAF\n\
+\t104 = JSON (flat)\n\n\
+\tValues 6 and 104 may be followed by a space-separated list of these keywords:\n\n\
 \tqseqid means Query Seq - id\n\
 \tqlen means Query sequence length\n\
 \tsseqid means Subject Seq - id\n\
@@ -382,29 +364,51 @@ Config::Config(int argc, const char **argv, bool check_io, CommandLineParser& pa
 \tqstrand means Query strand\n\
 \n\tDefault: qseqid sseqid pident length mismatch gapopen qstart qend sstart send evalue bitscore", output_format);
 
-	auto& cluster_opt = parser.add_group("Clustering options", { cluster, RECLUSTER, CLUSTER_REASSIGN, GREEDY_VERTEX_COVER, DEEPCLUST, LINCLUST });
+	auto& cluster_opt = parser.add_group("Clustering options", { cluster, RECLUSTER, DEEPCLUST, LINCLUST });
 	kmer_ranking = false;
 	cluster_opt.add()
-		("member-cover", 0, "Minimum coverage% of the cluster member sequence (default=80.0)", member_cover, 80.0)
 		("cluster-steps", 0, "Clustering steps", cluster_steps)
-		("centroid-out", 0, "Output file for centroids (greedy vertex cover workflow)", centroid_out)
 #ifdef KEEP_TARGET_ID
 		("kmer-ranking", 0, "Rank sequences based on kmer frequency in linear stage", kmer_ranking)
 #endif
 		;
 
-	auto& realign_opt = parser.add_group("Cluster input options", { CLUSTER_REALIGN, RECLUSTER, CLUSTER_REASSIGN, GREEDY_VERTEX_COVER });
-	realign_opt.add()
-		("clusters", 0, "Clustering input file mapping sequences to centroids", clustering)
+	auto& cluster_reassign_opt = parser.add_group("Clustering/reassign options", { cluster, RECLUSTER, CLUSTER_REASSIGN, GREEDY_VERTEX_COVER, DEEPCLUST, LINCLUST });
+	cluster_reassign_opt.add()
+		("memory-limit", 'M', "Memory limit in GB (default = 16G)", memory_limit)
+		("member-cover", 0, "Minimum coverage% of the cluster member sequence (default=80.0)", member_cover, 80.0);
+
+	auto& gvc_opt = parser.add_group("GVC options", { GREEDY_VERTEX_COVER });
+	gvc_opt.add()
+		("centroid-out", 0, "Output file for centroids", centroid_out)
 		("edges", 0, "Input file for greedy vertex cover", edges)
 		("edge-format", 0, "Edge format for greedy vertex cover (default/triplet)", edge_format);
 
+	auto& realign_opt = parser.add_group("Cluster input options", { CLUSTER_REALIGN, RECLUSTER, CLUSTER_REASSIGN });
+	realign_opt.add()
+		("clusters", 0, "Clustering input file mapping sequences to representatives", clustering);
+
 	string algo_str;
 
-	auto& advanced = parser.add_group("Advanced options", { blastp, blastx, makeidx, CLUSTER_REASSIGN, regression_test, cluster, DEEPCLUST, LINCLUST });
+	auto& advanced_gen = parser.add_group("Advanced/general", { blastp, blastx, blastn, CLUSTER_REASSIGN, regression_test, cluster, DEEPCLUST, LINCLUST, makedb });
+	advanced_gen.add()
+		("file-buffer-size", 0, "file buffer size in bytes (default=67108864)", file_buffer_size, (size_t)67108864)
+		("no-unlink", 0, "Do not unlink temporary files.", no_unlink)
+		("ignore-warnings", 0, "Ignore warnings", ignore_warnings)
+		("no-parse-seqids", 0, "Print raw seqids without parsing", no_parse_seqids);
+
+	auto& advanced_aln_cluster = parser.add_group("Advanced options aln/cluster", { blastp, blastx, blastn, CLUSTER_REASSIGN, regression_test, cluster, DEEPCLUST, LINCLUST, RECLUSTER });
+	advanced_aln_cluster.add()
+		("bin", 0, "number of query bins for seed search", query_bins_)
+		("ext-chunk-size", 0, "chunk size for adaptive ranking (default=auto)", ext_chunk_size)
+		("no-ranking", 0, "disable ranking heuristic", no_ranking)
+		("dbsize", 0, "effective database size (in letters)", db_size)
+		("no-auto-append", 0, "disable auto appending of DAA and DMND file extensions", no_auto_append)
+		("tantan-minMaskProb", 0, "minimum repeat probability for masking (default=0.9)", tantan_minMaskProb, 0.9);
+
+	auto& advanced = parser.add_group("Advanced options", { blastp, blastx, blastn, regression_test });
 	advanced.add()
 		("algo", 0, "Seed search algorithm (0=double-indexed/1=query-indexed/ctg=contiguous-seed)", algo_str)
-		("bin", 0, "number of query bins for seed search", query_bins_)
 		("min-orf", 'l', "ignore translated sequences without an open reading frame of at least this length", run_len)
 		("seed-cut", 0, "cutoff for seed complexity", seed_cut_)
 		("freq-masking", 0, "mask seeds based on frequency", freq_masking)
@@ -415,26 +419,18 @@ Config::Config(int argc, const char **argv, bool check_io, CommandLineParser& pa
 		("xdrop", 'x', "xdrop for ungapped alignment", ungapped_xdrop, 12.3)
 		("gapped-filter-evalue", 0, "E-value threshold for gapped filter (auto)", gapped_filter_evalue_, -1.0)
 		("band", 0, "band for dynamic programming computation", padding)
-		("shapes", 's', "number of seed shapes (default=all available)", shapes)
 		("shape-mask", 0, "seed shapes", shape_mask)
 		("multiprocessing", 0, "enable distributed-memory parallel processing", multiprocessing)
 		("mp-init", 0, "initialize multiprocessing run", mp_init)
 		("mp-recover", 0, "enable continuation of interrupted multiprocessing run", mp_recover)
 		("mp-query-chunk", 0, "process only a single query chunk as specified", mp_query_chunk, -1)
-		("ext-chunk-size", 0, "chunk size for adaptive ranking (default=auto)", ext_chunk_size)
-		("no-ranking", 0, "disable ranking heuristic", no_ranking)
 		("culling-overlap", 0, "minimum range overlap with higher scoring hit to delete a hit (default=50%)", inner_culling_overlap, 50.0)
 		("taxon-k", 0, "maximum number of targets to report per species", taxon_k, (uint64_t)0)
 		("range-cover", 0, "percentage of query range to be covered for range culling (default=50%)", query_range_cover, 50.0)
-		("dbsize", 0, "effective database size (in letters)", db_size)
-		("no-auto-append", 0, "disable auto appending of DAA and DMND file extensions", no_auto_append)
 		("xml-blord-format", 0, "Use gnl|BL_ORD_ID| style format in XML output", xml_blord_format)
-		("stop-match-score", 0, "Set the match score of stop codons against each other.", stop_match_score, 1)
-		("tantan-minMaskProb", 0, "minimum repeat probability for masking (default=0.9)", tantan_minMaskProb, 0.9)
-		("file-buffer-size", 0, "file buffer size in bytes (default=67108864)", file_buffer_size, (size_t)67108864)
-		("no-unlink", 0, "Do not unlink temporary files.", no_unlink)
+		("sam-query-len", 0, "add the query length to the SAM format (tag ZQ)", sam_qlen_field)
+		("stop-match-score", 0, "Set the match score of stop codons against each other.", stop_match_score, 1)		
 		("target-indexed", 0, "Enable target-indexed mode", target_indexed)
-		("ignore-warnings", 0, "Ignore warnings", ignore_warnings)
 		("unaligned-targets", 0, "", unaligned_targets)
 		("cut-bar", 0, "", cut_bar)
 		("check-multi-target", 0, "", check_multi_target)
@@ -444,12 +440,19 @@ Config::Config(int argc, const char **argv, bool check_io, CommandLineParser& pa
 		("query-parallel-limit", 0, "", query_parallel_limit, 3000000u)
 		("log-evalue-scale", 0, "", log_evalue_scale, 1.0 / std::log(2.0))
 		("bootstrap", 0, "", bootstrap)
+		("heartbeat", 0, "", heartbeat)
 		("mp-self", 0, "", mp_self)
+#ifdef EXTRA
+            ("zdrop", 'z', "zdrop for gapped dna alignment", zdrop, 40)
+#endif
 		("query-or-subject-cover", 0, "", query_or_target_cover);
 
-	auto& view_options = parser.add_group("View options", { view, blastp, blastx });
+	auto& view_align_options = parser.add_group("View/Align options", { view, blastp, blastx });
+	view_align_options.add()
+		("daa", 'a', "DIAMOND alignment archive (DAA) file", daa_file);
+
+	auto& view_options = parser.add_group("View options", { view });
 	view_options.add()
-		("daa", 'a', "DIAMOND alignment archive (DAA) file", daa_file)
 		("forwardonly", 0, "only show alignments of forward strand", forwardonly);
 
 	auto& getseq_options = parser.add_group("Getseq options", { getseq });
@@ -549,7 +552,6 @@ Config::Config(int argc, const char **argv, bool check_io, CommandLineParser& pa
 		("ungapped-evalue", 0, "", ungapped_evalue_, -1.0)
 		("ungapped-evalue-short", 0, "", ungapped_evalue_short_, -1.0)
 		("no-logfile", 0, "", no_logfile)
-		("no-heartbeat", 0, "", no_heartbeat)
 		("band-bin", 0, "", band_bin, 24)
 		("col-bin", 0, "", col_bin, 400)
 		("self", 0, "", self)
@@ -647,6 +649,7 @@ Config::Config(int argc, const char **argv, bool check_io, CommandLineParser& pa
 		("recluster_bd", 0, "", recluster_bd)
 		("pipeline-short", 0, "", pipeline_short)
 		("graph-algo", 0, "", graph_algo, string("gvc"))
+		("tsv-read-size", 0, "", tsv_read_size, int64_t(GIGABYTES))
 #ifndef KEEP_TARGET_ID
 		("kmer-ranking", 0, "Rank sequences based on kmer frequency in linear stage", kmer_ranking)
 #endif
@@ -654,6 +657,35 @@ Config::Config(int argc, const char **argv, bool check_io, CommandLineParser& pa
 
 	parser.store(argc, argv, command);
 
+	if (debug_log)
+		verbosity = 3;
+	else if (quiet)
+		verbosity = 0;
+	else if (verbose)
+		verbosity = 2;
+	else if (((command == Config::view || command == blastx || command == blastp || command == blastn) && output_file == "" && argc != 2)
+		|| command == Config::version || command == getseq || command == fastq2fasta || command == regression_test)
+		verbosity = 0;
+	else
+		verbosity = 1;
+
+	if (verbosity >= 1 || command == regression_test) {
+		ostream& header_out = command == Config::help ? cout : cerr;
+		header_out << Const::program_name << " v" << Const::version_string << "." << (unsigned)Const::build_version << " (C) Max Planck Society for the Advancement of Science, Benjamin Buchfink, University of Tuebingen" << endl;
+		header_out << "Documentation, support and updates available at http://www.diamondsearch.org" << endl;
+		header_out << "Please cite: http://dx.doi.org/10.1038/s41592-021-01101-x Nature Methods (2021)" << endl << endl;
+	}
+	log_stream << Const::program_name << " v" << Const::version_string << "." << (unsigned)Const::build_version << endl;
+
+    if(argc == 2 && command != version && command != regression_test) {
+        if (command != help) {
+            parser.print_documentation(command);
+        } else {
+            parser.print_help();
+        }
+    }
+
+
 	if (toppercent != 100.0 && max_target_seqs_.present())
 		throw std::runtime_error("--top and --max-target-seqs are mutually exclusive.");
 
@@ -671,6 +703,8 @@ Config::Config(int argc, const char **argv, bool check_io, CommandLineParser& pa
 	if (global_ranking_targets > 0 && (query_range_culling || taxon_k || multiprocessing || mp_init || mp_recover || comp_based_stats >= 2 || frame_shift > 0))
 		throw std::runtime_error("Global ranking is not supported in this mode.");
 
+
+
 #ifdef EXTRA
 	if (comp_based_stats >= Stats::CBS::COUNT)
 #else
@@ -683,14 +717,16 @@ Config::Config(int argc, const char **argv, bool check_io, CommandLineParser& pa
 	if (command == blastx && !Stats::CBS::support_translated(comp_based_stats))
 		throw std::runtime_error("This mode of composition based stats is not supported for translated searches.");
 
-	if (check_io) {
+    if (check_io) {
 		switch (command) {
-		case Config::makedb:
-			if (database == "")
-				throw std::runtime_error("Missing parameter: database file (--db/-d)");
-			if (chunk_size != 0.0)
-				throw std::runtime_error("Invalid option: --block-size/-b. Block size is set for the alignment commands.");
-			break;
+		case Config::makedb: {
+            if (database == "")
+                throw std::runtime_error("Missing parameter: database file (--db/-d)");
+            if (chunk_size != 0.0)
+                throw std::runtime_error(
+                        "Invalid option: --block-size/-b. Block size is set for the alignment commands.");
+            break;
+        }
 		case Config::blastp:
 		case Config::blastx:
         case Config::blastn:
@@ -721,18 +757,6 @@ Config::Config(int argc, const char **argv, bool check_io, CommandLineParser& pa
 		}
 	}
 
-	if (debug_log)
-		verbosity = 3;
-	else if (quiet)
-		verbosity = 0;
-	else if (verbose)
-		verbosity = 2;
-	else if (((command == Config::view || command == blastx || command == blastp || command == blastn) && output_file == "")
-		|| command == Config::version || command == getseq || command == fastq2fasta || command == regression_test)
-		verbosity = 0;
-	else
-		verbosity = 1;
-
 	switch (verbosity) {
 	case 0:
 		message_stream = MessageStream(false);
@@ -762,13 +786,7 @@ Config::Config(int argc, const char **argv, bool check_io, CommandLineParser& pa
 			auto_append_extension(output_file, ".zst");
 	}
 
-	if (verbosity >= 1 || command == regression_test) {
-		ostream &header_out = command == Config::help ? cout : cerr;
-		header_out << Const::program_name << " v" << Const::version_string << "." << (unsigned)Const::build_version << " (C) Max Planck Society for the Advancement of Science" << endl;
-		header_out << "Documentation, support and updates available at http://www.diamondsearch.org" << endl;
-		header_out << "Please cite: http://dx.doi.org/10.1038/s41592-021-01101-x Nature Methods (2021)" << endl << endl;
-	}
-	log_stream << Const::program_name << " v" << Const::version_string << "." << (unsigned)Const::build_version << endl;
+	
 #ifndef NDEBUG
 	verbose_stream << "Assertions enabled." << endl;
 #endif
@@ -788,7 +806,8 @@ Config::Config(int argc, const char **argv, bool check_io, CommandLineParser& pa
 	case Config::CLUSTER_REASSIGN:
 	case Config::GREEDY_VERTEX_COVER:
 	case Config::RECLUSTER:
-		message_stream << "#CPU threads: " << threads_ << endl;
+		if (argc != 2)
+			message_stream << "#CPU threads: " << threads_ << endl;
 	default:
 		;
 	}
@@ -810,6 +829,7 @@ Config::Config(int argc, const char **argv, bool check_io, CommandLineParser& pa
 	case Config::CLUSTER_REASSIGN:
 	case Config::CLUSTER_REALIGN:
 	case Config::RECLUSTER:
+	case Config::MODEL_SEQS:
 		if (frame_shift != 0 && command == Config::blastp)
 			throw std::runtime_error("Frameshift alignments are only supported for translated searches.");
 		if (query_range_culling && frame_shift == 0)
@@ -826,7 +846,8 @@ Config::Config(int argc, const char **argv, bool check_io, CommandLineParser& pa
 				throw std::runtime_error("This value for --comp-based-stats is not supported when using a custom scoring matrix.");
 			score_matrix = ScoreMatrix(matrix_file, gap_open, gap_extend, stop_match_score, ScoreMatrix::Custom());
 		}
-		message_stream << "Scoring parameters: " << score_matrix << endl;
+		if (argc != 2)
+			message_stream << "Scoring parameters: " << score_matrix << endl;
 		Masking::instance = unique_ptr<Masking>(new Masking(score_matrix));
 	}
 
@@ -838,11 +859,6 @@ Config::Config(int argc, const char **argv, bool check_io, CommandLineParser& pa
 
 		raw_ungapped_xdrop = score_matrix.rawscore(ungapped_xdrop);
 		verbose_stream << "CPU features detected: " << SIMD::features() << endl;
-#if defined(RUNTIME_DISPATCH) && defined(__SSE__)
-		verbose_stream << "Runtime dispatch: enabled" << endl;
-#else
-		verbose_stream << "Runtime dispatch: disabled" << endl;
-#endif
 		log_stream << "L3 cache size: " << l3_cache_size() << endl;
 	}
 
@@ -862,10 +878,6 @@ Config::Config(int argc, const char **argv, bool check_io, CommandLineParser& pa
 	if (command == blastx || command == blastn)
 		input_value_traits = nucleotide_traits;
 
-
-	if (command == help)
-		parser.print_help();
-
 	if (query_strands != "both" && query_strands != "minus" && query_strands != "plus")
 		throw std::runtime_error("Invalid value for parameter --strand");
 
@@ -927,4 +939,4 @@ Config::Config(int argc, const char **argv, bool check_io, CommandLineParser& pa
 	log_stream << " KEEP_TARGET_ID";
 #endif
 	log_stream << endl;
-}
+}
\ No newline at end of file
