@@ -33,7 +33,6 @@ using std::pair;
 using std::endl;
 using std::vector;
 using std::ofstream;
-using std::cout;
 using std::ostream;
 using std::runtime_error;
 using std::string;
@@ -213,12 +212,12 @@ void init_thresholds() {
 	}
 	else {
 		config.diag_filter_id = 85.0;
-		config.diag_filter_cov = config.member_cover > 50.0 ? config.member_cover - 5.0 : 0.0;
+		config.diag_filter_cov = config.member_cover > 50.0 ? config.member_cover - 10.0 : 0.0;
 	}
 }
 
 File* open_out_tsv() {
-	File* file = new File(Schema{ Type::STRING, Type::STRING }, config.output_file, Flags::WRITE | Flags::OVERWRITE);
+	File* file = new File(Schema{ Type::STRING, Type::STRING }, config.output_file, Flags::WRITE);
 	if (Blast_tab_format::header_format(::Config::cluster) == Header::SIMPLE)
 		file->write_record("centroid", "member");
 	return file;
