@@ -44,7 +44,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 using std::tuple;
 using std::string;
 using std::list;
-using std::cout;
 using std::endl;
 using std::unique_ptr;
 using std::pair;
@@ -235,8 +234,8 @@ void DatabaseFile::make_db()
 	else
 		message_stream << "Database input file: " << input_file_name << endl;
 
-	task_timer total;
-	task_timer timer("Opening the database file", true);
+	TaskTimer total;
+	TaskTimer timer("Opening the database file", true);
 
 	value_traits = (config.dbtype == SequenceType::amino_acid) ? amino_acid_traits : nucleotide_traits;
     FastaFile db_file({ input_file_name }, Metadata (), Flags::NONE, value_traits);
@@ -261,6 +260,7 @@ void DatabaseFile::make_db()
 	const FASTA_format format;
 	vector<SeqInfo> pos_array;
 	ExternalSorter<pair<string, OId>> accessions;
+	AccessionParsing acc_stats;
 	try {
 		while (true) {
 			timer.go("Loading sequences");
@@ -286,7 +286,7 @@ void DatabaseFile::make_db()
 			if (!config.prot_accession2taxid.empty()) {
 				timer.go("Writing accessions");
 				for (size_t i = 0; i < n; ++i) {
-					vector<string> acc = accession_from_title(block->ids()[i]);
+					vector<string> acc = accession_from_title(block->ids()[i], acc_stats);
 					for (const string& s : acc)
 						accessions.push(std::make_pair(s, total_seqs + i));
 				}
@@ -318,9 +318,13 @@ void DatabaseFile::make_db()
 	pos_array.shrink_to_fit();
 	timer.finish();
 
+	if (!config.prot_accession2taxid.empty() && !config.no_parse_seqids)
+		message_stream << endl << "Accession parsing rules triggered for database seqids (use --no-parse-seqids to disable):" << endl << acc_stats << endl;
+
 	Util::Table stats;
 	stats("Database sequences", n_seqs);
 	stats("Database letters", letters);
+
 	taxonomy.init();
 	if (!config.prot_accession2taxid.empty()) {
 		header2.taxon_array_offset = out->tell();
@@ -398,7 +402,7 @@ void DatabaseFile::skip_seq()
 }
 
 bool DatabaseFile::is_diamond_db(const string &file_name) {
-	if (file_name == "-")
+	if (file_name == "-" || file_name.empty())
 		return false;
 	InputFile db_file(file_name);
 	uint64_t magic_number = 0;
@@ -413,7 +417,7 @@ bool DatabaseFile::is_diamond_db(const string &file_name) {
 
 void DatabaseFile::create_partition_fixednumber(size_t n) {
 	size_t max_letters_balanced = static_cast<size_t>(std::ceil(static_cast<double>(ref_header.letters)/static_cast<double>(n)));
-	cout << "Fixed number partitioning using " << max_letters_balanced << " (" << n << ")" << endl;
+	message_stream << "Fixed number partitioning using " << max_letters_balanced << " (" << n << ")" << endl;
 	this->create_partition(max_letters_balanced);
 }
 
@@ -425,7 +429,7 @@ void DatabaseFile::create_partition_balanced(size_t max_letters) {
 }
 
 void DatabaseFile::create_partition(size_t max_letters) {
-	task_timer timer("Create partition of DatabaseFile");
+	TaskTimer timer("Create partition of DatabaseFile");
 	size_t letters = 0, seqs = 0, total_seqs = 0;
 	int i_chunk = 0;
 
@@ -647,7 +651,7 @@ const char* const SEQID_EXTENSION = ".seqid";
 
 void DatabaseFile::prep_db() {
 	config.database.require();
-	task_timer timer("Opening the database");
+	TaskTimer timer("Opening the database");
 	DatabaseFile db(config.database);
 	timer.go("Writing seqid list");
 	db.make_seqid_list(); // db.file_name() + SEQID_EXTENSION);
@@ -666,6 +670,9 @@ void DatabaseFile::read_seqid_list() {
 	init_seq_access();
 	for (int64_t n = 0; n < sequence_count(); ++n) {
 		read_seq(seq, id);
+		const char* msg = Util::Seq::fix_title(id);
+		if (msg)
+			message_stream << "Warning: " << msg << std::endl;
 		add_seqid_mapping(id, oid++);
 	}
 	/*if ((flag_any(flags_, Flags::ACC_TO_OID_MAPPING) && (int64_t)acc2oid_.size() != sequence_count())
