@@ -55,30 +55,42 @@ static int mapping_file_format(const string& header) {
 	throw std::runtime_error("Accession mapping file header has to be in one of these formats:\naccession\taccession.version\ttaxid\tgi\naccession.version\ttaxid");
 }
 
-static void load_mapping_file(ExternalSorter<pair<string, TaxId>>& sorter)
+static AccessionParsing load_mapping_file(ExternalSorter<pair<string, TaxId>>& sorter)
 {
 	TaxId taxid;
 	TextInputFile f(config.prot_accession2taxid);
 	f.getline();
 	int format = mapping_file_format(f.line);
 	string accession, last;
+	AccessionParsing stats;
 
 	while (!f.eof() && (f.getline(), !f.line.empty())) {
-		if (format == 0)
-			Util::String::Tokenizer(f.line, "\t") >> Util::String::Skip() >> accession >> taxid;
-		else
-			Util::String::Tokenizer(f.line, "\t") >> accession >> taxid;
+		try {
+			if (format == 0)
+				Util::String::Tokenizer(f.line, "\t") >> Util::String::Skip() >> accession >> taxid;
+			else
+				Util::String::Tokenizer(f.line, "\t") >> accession >> taxid;
+		}
+		catch (Util::String::TokenizerException&) {
+			throw std::runtime_error("Malformed input in line " + std::to_string(f.line_count));
+		}
 
 		if (accession.empty())
 			throw std::runtime_error("Empty accession field in line " + std::to_string(f.line_count));
 
-		size_t i = accession.find(":PDB=");
-		if (i != string::npos)
-			accession.erase(i);
+		if (!config.no_parse_seqids) {
+			size_t i = accession.find(":PDB=");
+			if (i != string::npos) {
+				accession.erase(i);
+				++stats.pdb_suffix;
+			}
 
-		i = accession.find_last_of('.');
-		if (i != string::npos)
-			accession.erase(i);
+			i = accession.find_last_of('.');
+			if (i != string::npos) {
+				accession.erase(i);
+				++stats.suffix_after_dot;
+			}
+		}
 
 		if (accession != last)
 			sorter.push(make_pair(accession, taxid));
@@ -86,13 +98,14 @@ static void load_mapping_file(ExternalSorter<pair<string, TaxId>>& sorter)
 		last = accession;
 	}
 	f.close();
+	return stats;
 }
 
 void TaxonList::build(OutputFile &db, ExternalSorter<pair<string, OId>>& acc2oid, OId seqs, Util::Table& stats)
 {
-	task_timer timer("Loading taxonomy mapping file");
+	TaskTimer timer("Loading taxonomy mapping file");
 	ExternalSorter<pair<string, TaxId>> acc2taxid;
-	load_mapping_file(acc2taxid);
+	const AccessionParsing acc_stats = load_mapping_file(acc2taxid);
 
 	timer.go("Joining accession mapping");
 	acc2taxid.init_read();
@@ -128,4 +141,7 @@ void TaxonList::build(OutputFile &db, ExternalSorter<pair<string, OId>>& acc2oid
 	stats("Entries in accession to taxid file", acc2taxid.count());
 	stats("Database accessions mapped to taxid" , acc_matched);
 	stats("Database sequences mapped to taxid", mapped_seqs);
+
+	if (!config.no_parse_seqids)
+		message_stream << endl << "Accession parsing rules triggered for mapping file seqids (use --no-parse-seqids to disable):" << endl << acc_stats << endl;
 }
