@@ -27,17 +27,17 @@ void PAF_format::print_query_intro(Output::Info& info) const
 	}
 }
 
-void PAF_format::print_match(const HspContext& r, const Search::Config& cfg, TextBuffer &out)
+void PAF_format::print_match(const HspContext& r, Output::Info& info)
 {
-	out.write_until(r.query_title.c_str(), Util::Seq::id_delimiters);
-	out << '\t' << r.query.source().length() << '\t'
+	info.out.write_until(r.query_title.c_str(), Util::Seq::id_delimiters);
+	info.out << '\t' << r.query.source().length() << '\t'
 		<< r.query_source_range().begin_ << '\t'
 		<< r.query_source_range().end_ - 1 << '\t'
 		<< (Frame(r.frame()).strand == FORWARD ? '+' : '-') << '\t';
 
-	print_title(out, r.target_title.c_str(), false, false, "<>");
+	print_title(info.out, r.target_title.c_str(), false, false, "<>");
 
-	out << '\t' << r.subject_len << '\t'
+	info.out << '\t' << r.subject_len << '\t'
 		<< r.subject_range().begin_ << '\t'
 		<< r.subject_range().end_ - 1 << '\t'
 		<< r.identities() << '\t'
@@ -46,6 +46,6 @@ void PAF_format::print_match(const HspContext& r, const Search::Config& cfg, Tex
 		<< "AS:i:" << (uint32_t)score_matrix.bitscore(r.score()) << '\t'
 		<< "ZR:i:" << r.score() << '\t'
 		<< "ZE:f:";
-	out.print_e(r.evalue());
-	out << '\n';
+	info.out.print_e(r.evalue());
+	info.out << '\n';
 }
\ No newline at end of file
