@@ -54,6 +54,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "../util/async_buffer.h"
 #include "config.h"
 #include "../data/seed_array.h"
+#include "../data/fasta/fasta_file.h"
 #ifdef WITH_DNA
 #include "../dna/dna_index.h"
 #endif
@@ -103,7 +104,7 @@ static void run_ref_chunk(SequenceFile &db_file,
 	PtrVector<TempFile> &tmp_file,
 	Config& cfg)
 {
-	task_timer timer;
+	TaskTimer timer;
 	log_rss();
 	auto& query_seqs = cfg.query->seqs();
 
@@ -150,16 +151,22 @@ static void run_ref_chunk(SequenceFile &db_file,
 
 	if (!config.swipe_all) {
 		timer.go("Building reference histograms");
-		if(query_seeds_bitset.get())
-			cfg.target->hst() = SeedHistogram(*cfg.target, true, query_seeds_bitset.get(), cfg.seed_encoding, nullptr, false, cfg.seed_complexity_cut, MaskingAlgo::NONE, cfg.minimizer_window);
-		else if (query_seeds_hashed.get())
-			cfg.target->hst() = SeedHistogram(*cfg.target, true, query_seeds_hashed.get(), cfg.seed_encoding, nullptr, false, cfg.seed_complexity_cut, MaskingAlgo::NONE, cfg.minimizer_window);
-		else
-			cfg.target->hst() = SeedHistogram(*cfg.target, false, &no_filter, cfg.seed_encoding, nullptr, false, cfg.seed_complexity_cut, cfg.soft_masking, cfg.minimizer_window);
+		if (query_seeds_bitset.get()) {
+			EnumCfg enum_cfg{ nullptr, 0, 0, cfg.seed_encoding, nullptr, false, false, cfg.seed_complexity_cut, MaskingAlgo::NONE, cfg.minimizer_window, false, false };
+			cfg.target->hst() = SeedHistogram(*cfg.target, true, query_seeds_bitset.get(), enum_cfg);
+		}
+		else if (query_seeds_hashed.get()) {
+			EnumCfg enum_cfg{ nullptr, 0, 0, cfg.seed_encoding, nullptr, false, false, cfg.seed_complexity_cut, MaskingAlgo::NONE, cfg.minimizer_window, false, false };
+			cfg.target->hst() = SeedHistogram(*cfg.target, true, query_seeds_hashed.get(), enum_cfg);
+		}
+		else {
+			EnumCfg enum_cfg{ nullptr, 0, 0, cfg.seed_encoding, nullptr, false, false, cfg.seed_complexity_cut, cfg.soft_masking, cfg.minimizer_window, false, false };
+			cfg.target->hst() = SeedHistogram(*cfg.target, false, &no_filter, enum_cfg);
+		}
 
 		timer.go("Allocating buffers");
 		char* ref_buffer = SeedArray::alloc_buffer(cfg.target->hst(), cfg.index_chunks),
-			* query_buffer = SeedArray::alloc_buffer(cfg.query->hst(), cfg.index_chunks);
+			* query_buffer = config.target_indexed ? nullptr : SeedArray::alloc_buffer(cfg.query->hst(), cfg.index_chunks);
 		timer.finish();
 
 		::HashedSeedSet* target_seeds = nullptr;
@@ -179,17 +186,22 @@ static void run_ref_chunk(SequenceFile &db_file,
         }
 #ifdef WITH_DNA
         else
-            cfg.dna_ref_index = std::make_unique<Dna::Index>(cfg,ref_buffer);
+            cfg.dna_ref_index = std::make_unique<Dna::Index>(cfg, ref_buffer);
 #endif
-      
 
-		timer.go("Deallocating buffers");
+        log_rss();
+        timer.go("Deallocating buffers");
+#ifdef WITH_DNA
+        if(config.command != ::Config::blastn)
+#endif
 		delete[] ref_buffer;
 		delete[] query_buffer;
 		delete target_seeds;
 
 		timer.go("Clearing query masking");
 		Frequent_seeds::clear_masking(query_seqs);
+		timer.finish();
+		log_rss();
 	}
 
     Consumer* out;
@@ -234,7 +246,7 @@ static void run_query_iteration(const unsigned query_iteration,
 	PtrVector<TempFile>& tmp_file,
 	Config& options)
 {
-	task_timer timer;
+	TaskTimer timer;
 	auto P = Parallelizer::get();
 	auto& query_seqs = options.query->seqs();
 	auto& db_file = *options.db;
@@ -279,7 +291,6 @@ static void run_query_iteration(const unsigned query_iteration,
 		options.cutoff_gapped2_new = { options.gapped_filter_evalue };
 	}
 
-
 	if (options.current_query_block == 0 && query_iteration == 0) {
 		message_stream << "Algorithm: " << to_string(config.algo) << endl;
 		if (config.freq_masking && !config.lin_stage1)
@@ -294,7 +305,9 @@ static void run_query_iteration(const unsigned query_iteration,
 
 	if (!config.swipe_all && !config.target_indexed) {
 		timer.go("Building query histograms");
-		options.query->hst() = SeedHistogram(*options.query, false, &no_filter, options.seed_encoding, options.query_skip.get(), false, options.seed_complexity_cut, options.soft_masking, options.minimizer_window);
+		EnumCfg enum_cfg{ nullptr, 0, 0, options.seed_encoding, options.query_skip.get(), false, false, options.seed_complexity_cut,
+			options.soft_masking, options.minimizer_window, (bool)query_seeds_hashed, false };
+		options.query->hst() = SeedHistogram(*options.query, false, &no_filter, enum_cfg);
 		timer.finish();
 	}
 
@@ -363,12 +376,17 @@ static void run_query_iteration(const unsigned query_iteration,
 			db_file.set_seqinfo_ptr(options.query->oid_end());
 		else if (!config.self || options.current_query_block != 0 || !db_file.eof())
 			db_file.set_seqinfo_ptr(0);*/
+		timer.go("Seeking in database");
 		db_file.set_seqinfo_ptr((config.self && !config.lin_stage1) ? options.query->oid_end() : 0);
+		timer.finish();
 		for (options.current_ref_block = 0; ; ++options.current_ref_block) {
 			if (config.self && ((config.lin_stage1 && options.current_ref_block == options.current_query_block) || (!config.lin_stage1 && options.current_ref_block == 0))) {
 				options.target = options.query;
-				if (config.lin_stage1)
+				if (config.lin_stage1) {
+					timer.go("Seeking in database");
 					db_file.set_seqinfo_ptr(options.query->oid_end());
+					timer.finish();
+				}
 			}
 			else {
 				timer.go("Loading reference sequences");
@@ -412,7 +430,7 @@ static void run_query_chunk(Consumer &master_out,
 	Config &options)
 {
 	auto P = Parallelizer::get();
-	task_timer timer;
+	TaskTimer timer;
 	auto& db_file = *options.db;
 	auto& query_seqs = options.query->seqs();
 
@@ -520,7 +538,7 @@ static void run_query_chunk(Consumer &master_out,
 	options.query.reset();
 }
 
-static void master_thread(task_timer &total_timer, Config &options)
+static void master_thread(TaskTimer &total_timer, Config &options)
 {
 	log_rss();
 	SequenceFile* db_file = options.db.get();
@@ -571,12 +589,14 @@ static void master_thread(task_timer &total_timer, Config &options)
 		db_file->create_partition_balanced((size_t)(config.chunk_size*1e9));
 	}
 
-	task_timer timer("Opening the input file", true);
+	TaskTimer timer("Opening the input file", true);
 	if (!options.self) {
-		if (config.query_file.empty() && !options.query_file)
+		if (config.query_file.empty() && !options.query_file) {
 			std::cerr << "Query file parameter (--query/-q) is missing. Input will be read from stdin." << endl;
+			config.query_file.push_back("");
+		}
 		if (!options.query_file)
-			options.query_file.reset(SequenceFile::auto_create(config.query_file, SequenceFile::Flags(), SequenceFile::Metadata(), input_value_traits));
+			options.query_file.reset(new FastaFile(config.query_file, SequenceFile::Metadata(), SequenceFile::Flags(), input_value_traits));
 	}
 
 	options.current_query_block = 0;
@@ -587,7 +607,7 @@ static void master_thread(task_timer &total_timer, Config &options)
 	if (config.command == ::Config::blastn)
 		load_flags |= SequenceFile::LoadFlags::DNA_PRESERVATION;
 	if (config.multiprocessing && config.mp_init) {
-		task_timer timer("Counting query blocks", true);
+		TaskTimer timer("Counting query blocks", true);
 
 		size_t block_count = 0;
 		do {
@@ -628,19 +648,23 @@ static void master_thread(task_timer &total_timer, Config &options)
 
 	for (;query_file_offset < db_file->sequence_count(); ++options.current_query_block) {
 		log_rss();
-		timer.go("Loading query sequences");
 
 		if (options.self) {
+			timer.go("Seeking in database");
 			db_file->set_seqinfo_ptr(query_file_offset);
+			timer.finish();
+			timer.go("Loading query sequences");
 			options.query.reset(db_file->load_seqs(config.block_size(), options.db_filter.get(), SequenceFile::LoadFlags::ALL));
 			query_file_offset = db_file->tell_seq();
 		}
-		else
+		else {
+			timer.go("Loading query sequences");
 			options.query.reset(options.query_file->load_seqs(config.block_size(), nullptr, load_flags));
+		}
+		timer.finish();
 
 		if (options.query->empty())
 			break;
-		timer.finish();
 		options.query->seqs().print_stats();
 		if ((config.mp_query_chunk >= 0) && (options.current_query_block != config.mp_query_chunk))
 			continue;
@@ -710,7 +734,7 @@ static void master_thread(task_timer &total_timer, Config &options)
 
 void run(const shared_ptr<SequenceFile>& db, const shared_ptr<SequenceFile>& query, const shared_ptr<Consumer>& out, const shared_ptr<BitVector>& db_filter)
 {
-	task_timer total;
+	TaskTimer total;
 
 	align_mode = AlignMode(AlignMode::from_command(config.command));
     (align_mode.sequence_type == SequenceType::amino_acid) ? value_traits = amino_acid_traits : value_traits = nucleotide_traits;
@@ -738,7 +762,7 @@ void run(const shared_ptr<SequenceFile>& db, const shared_ptr<SequenceFile>& que
 	if (cfg.output_format->needs_taxon_ranks || taxon_culling)
 		metadata_flags |= SequenceFile::Metadata::TAXON_RANKS;
 
-	task_timer timer;
+	TaskTimer timer;
 	SequenceFile::Flags flags(SequenceFile::Flags::NEED_LETTER_COUNT);
 	if (flag_any(cfg.output_format->flags, Output::Flags::ALL_SEQIDS))
 		flags |= SequenceFile::Flags::ALL_SEQIDS;
