@@ -39,6 +39,8 @@ bool Search::seed_is_complex(const Letter* seq, const Shape& shape, const double
 	count.fill(0);
 	for (int i = 0; i < shape.weight_; ++i) {
 		const Letter l = letter_mask(seq[shape.positions_[i]]);
+		if (l >= TRUE_AA)
+			return false;
 		++count[Reduction::reduction(l)];
 	}
 	double entropy = lnfact[shape.weight_];
@@ -76,7 +78,7 @@ void Search::mask_seeds(const Shape& shape, const SeedPartitionRange& range, Dou
 	if (cfg.seed_encoding != SeedEncoding::SPACED_FACTOR)
 		return;
 
-	task_timer timer("Masking low complexity seeds");
+	TaskTimer timer("Masking low complexity seeds");
 	SequenceSet& query_seqs = cfg.query->seqs();
 	std::atomic_int seedp(range.begin());
 	std::atomic_size_t seed_count(0), masked_seed_count(0), query_count(0), target_count(0);
