@@ -79,18 +79,10 @@ static void search_worker(atomic<unsigned> *seedp, const SeedPartitionRange *see
 	unique_ptr<Search::WorkSet> work_set(new Search::WorkSet{ *context, *cfg, shape, {}, writer.get(), {}, {}, {}, context->kmer_ranking });
 #endif
 	int p;
-#ifdef KEEP_TARGET_ID
-	auto f = config.lin_stage1 ? Search::stage1_query_lin_ranked
-		: (cfg->lin_stage1_target ? Search::stage1_target_lin
-			: (config.self && cfg->current_ref_block == 0 ? Search::stage1_self : Search::stage1));
-#else
-	auto f = config.lin_stage1 ? Search::stage1_query_lin
-		: (cfg->lin_stage1_target ? Search::stage1_target_lin
-			: (config.self && cfg->current_ref_block == 0 ? Search::stage1_self : Search::stage1));
-#endif
-	while ((p = (*seedp)++) < seedp_range->end())
-		for (auto it = JoinIterator<SeedLoc>(query_seed_hits[p].begin(), ref_seed_hits[p].begin()); it; ++it)
-			f(it.r->begin(), (int32_t)it.r->size(), it.s->begin(), (int32_t)it.s->size(), *work_set);
+	while ((p = (*seedp)++) < seedp_range->end()) {
+		auto it = JoinIterator<SeedLoc>(query_seed_hits[p].begin(), ref_seed_hits[p].begin());
+		run_stage1(it, work_set.get(), cfg);
+	}
 	statistics += work_set->stats;
 }
 
@@ -114,11 +106,11 @@ void search_shape(unsigned sid, int query_block, unsigned query_iteration, char
 		const SeedPartitionRange range(p.begin(chunk), p.end(chunk));
 		current_range = range;
 
-		task_timer timer("Building reference seed array", true);
+		TaskTimer timer("Building reference seed array", true);
 		SeedArray *ref_idx;
 		const EnumCfg enum_ref{ &ref_hst.partition(), sid, sid + 1, cfg.seed_encoding, nullptr, false, false, cfg.seed_complexity_cut,
-			query_seeds_bitset.get() || query_seeds_hashed.get() ? MaskingAlgo::NONE : cfg.soft_masking,
-			cfg.minimizer_window };
+			query_seeds_bitset.get() || (bool)query_seeds_hashed ? MaskingAlgo::NONE : cfg.soft_masking,
+			cfg.minimizer_window, false, false };
 		if (query_seeds_bitset.get())
 			ref_idx = new SeedArray(*cfg.target, ref_hst.get(sid), range, ref_buffer, query_seeds_bitset.get(), enum_ref);
 		else if (query_seeds_hashed.get())
@@ -126,15 +118,19 @@ void search_shape(unsigned sid, int query_block, unsigned query_iteration, char
 			//ref_idx = new SeedArray(ref_seqs, sid, range, query_seeds_hashed.get(), true);
 		else
 			ref_idx = new SeedArray(*cfg.target, ref_hst.get(sid), range, ref_buffer, &no_filter, enum_ref);
+		timer.finish();
+		log_rss();
 
 		timer.go("Building query seed array");
 		SeedArray* query_idx;
-		EnumCfg enum_query{ target_seeds ? nullptr : &query_hst.partition(), sid, sid + 1, cfg.seed_encoding, cfg.query_skip.get(), false, true, cfg.seed_complexity_cut, cfg.soft_masking, cfg.minimizer_window };
+		EnumCfg enum_query{ target_seeds ? nullptr : &query_hst.partition(), sid, sid + 1, cfg.seed_encoding, cfg.query_skip.get(),
+			false, true, cfg.seed_complexity_cut, cfg.soft_masking, cfg.minimizer_window, (bool)query_seeds_hashed, (bool)query_seeds_hashed };
 		if (target_seeds)
 			query_idx = new SeedArray(*cfg.query, range, target_seeds, enum_query);
 		else
 			query_idx = new SeedArray(*cfg.query, query_hst.get(sid), range, query_buffer, &no_filter, enum_query);
 		timer.finish();
+		log_rss();
 
 		log_stream << "Indexed query seeds = " << Util::String::ratio_percentage(query_idx->size(), query_seqs.letters())
 			<< ", reference seeds = " << Util::String::ratio_percentage(ref_idx->size(), ref_seqs.letters()) << endl;
@@ -151,14 +147,16 @@ void search_shape(unsigned sid, int query_block, unsigned query_iteration, char
 		for (auto &t : threads)
 			t.join();
 		timer.finish();
+		log_rss();
 
-		if(config.freq_masking && !config.lin_stage1) {
+		if(config.freq_masking && !config.lin_stage1 && !cfg.lin_stage1_target) {
 			timer.go("Building seed filter");
 			frequent_seeds.build(sid, range, query_seed_hits, ref_seed_hits, cfg);
 		}
 		else
 			Search::mask_seeds(shapes[sid], range, query_seed_hits, ref_seed_hits, cfg);
 
+		log_rss();
 		unique_ptr<KmerRanking> kmer_ranking;
 #ifdef KEEP_TARGET_ID
 		if (config.lin_stage1) {
@@ -183,12 +181,16 @@ void search_shape(unsigned sid, int query_block, unsigned query_iteration, char
 			threads.emplace_back(search_worker, &seedp, &range, sid, i, query_seed_hits, ref_seed_hits, context, &cfg);
 		for (auto &t : threads)
 			t.join();
+		timer.finish();
+		log_rss();
 
 		timer.go("Deallocating memory");
 		delete ref_idx;
 		delete query_idx;
 		delete context;
 		kmer_ranking.reset();
+		timer.finish();
+		log_rss();
 	}
 }
 
